@@ -1,12 +1,14 @@
-__version__ = "0.17.1"
+__version__ = "0.31.0"
 
 
 from .ast import *
 from .codegen import *
+from .emit import *
 from .helpers import *
 from .loop_info import *
-from .macro import *
+from .memo import *
 from .module import *
 from .parse import *
 from .plugin import *
 from .runtime import *
+from .semantics import *
