@@ -22,6 +22,7 @@ from typing import (
     List,
     Literal,
     Optional,
+    Set,
     Tuple,
     Type,
     Union,
@@ -47,11 +48,18 @@ from .ast import (
     AstClassName,
     AstDict,
     AstDictItem,
+    AstDocstring,
+    AstEscapeRoot,
     AstExpressionBinary,
     AstExpressionUnary,
     AstFormatString,
     AstFromImport,
     AstFunctionSignature,
+    AstFunctionSignatureArgument,
+    AstFunctionSignaturePositionalMarker,
+    AstFunctionSignatureVariadicArgument,
+    AstFunctionSignatureVariadicKeywordArgument,
+    AstFunctionSignatureVariadicMarker,
     AstIdentifier,
     AstImportedItem,
     AstImportedMacro,
@@ -62,6 +70,8 @@ from .ast import (
     AstMacro,
     AstMacroCall,
     AstMacroMatchArgument,
+    AstMemo,
+    AstPrelude,
     AstProcMacro,
     AstSlice,
     AstTarget,
@@ -70,6 +80,7 @@ from .ast import (
     AstTargetItem,
     AstTargetUnpack,
     AstTuple,
+    AstTypeDeclaration,
     AstUnpack,
     AstValue,
 )
@@ -82,8 +93,11 @@ class Accumulator:
 
     indentation: str = ""
     refs: List[Any] = field(default_factory=list)
+    dependencies: Set[str] = field(default_factory=set)
+    prelude_imports: List[AstPrelude] = field(default_factory=list)
     macros: MacroLibrary = field(default_factory=dict)
     macro_ids: Dict[str, int] = field(default_factory=dict)
+    memo_index: Dict[AstMemo, int] = field(default_factory=dict)
     lines: List[str] = field(default_factory=list)
     counter: int = 0
     header: Dict[str, str] = field(default_factory=dict)
@@ -212,9 +226,10 @@ class Accumulator:
         self.lines.append(f"{self.indentation}{code}\n")
 
     @contextmanager
-    def function(self, name: str, *args: str):
+    def function(self, name: str, *args: str, return_type: str = ""):
         """Emit function."""
-        self.statement(f"def {name}({', '.join(args)}):")
+        return_type_annotation = return_type and f" -> {return_type}"
+        self.statement(f"def {name}({', '.join(args)}){return_type_annotation}:")
         with self.block():
             previous_root = self.root_scope
             self.root_scope = False
@@ -342,7 +357,7 @@ def visit_multiple(
 ) -> Generator[AstNode, Optional[List[str]], Optional[str]]:
     """Yield all the nodes and return a single result pointing to the new children."""
     current_count = 0
-    collector = None
+    collector: Optional[ChildrenCollector] = None
     index = len(acc.lines)
 
     for i, child in enumerate(children):
@@ -460,6 +475,8 @@ class Codegen(Visitor):
             source=acc.get_source(),
             output=output,
             refs=acc.refs,
+            dependencies=acc.dependencies,
+            prelude_imports=acc.prelude_imports,
             macros=acc.macros,
         )
 
@@ -494,6 +511,64 @@ class Codegen(Visitor):
         yield node.arguments[0]
         return []
 
+    @rule(AstMemo)
+    def memo(
+        self,
+        node: AstMemo,
+        acc: Accumulator,
+    ) -> Generator[AstNode, Optional[List[str]], Optional[List[str]]]:
+        file_index = acc.memo_index.setdefault(node, 0)
+        acc.memo_index[node] = file_index + 1
+
+        keys: List[str] = []
+
+        *arguments, body = node.arguments
+        cached_identifiers = (
+            "".join(f"{name}," for name in body.identifiers)
+            if isinstance(body, AstEscapeRoot)
+            else None
+        )
+
+        for arg in arguments:
+            if isinstance(arg, AstAssignment):
+                target = cast(AstTargetIdentifier, arg.target)
+                yield arg
+                keys.append(f"{target.value},")
+            else:
+                value = yield from visit_single(arg, required=True)
+                keys.append(f"{value},")
+
+        storage = f"_bolt_memo_storage_{node.persistent_id.hex}"
+        acc.header[storage] = "None"
+        if not acc.root_scope:
+            acc.statement(f"global {storage}", lineno=node)
+        acc.statement(f"if {storage} is None:")
+        with acc.block():
+            acc.statement(
+                f"{storage} = _bolt_runtime.memo.registry[__file__][{acc.make_ref(node)}, {file_index}]"
+            )
+
+        invocation = f"_bolt_memo_invocation_{node.persistent_id.hex}"
+        acc.statement(f"{invocation} = {storage}[({' '.join(keys)})]")
+
+        acc.statement(f"if {invocation}.cached:")
+        with acc.block():
+            acc.statement(f"_bolt_runtime.memo.restore(_bolt_runtime, {invocation})")
+            if cached_identifiers:
+                acc.statement(f"({cached_identifiers}) = {invocation}.bindings")
+
+        acc.statement("else:")
+        with acc.block():
+            acc.statement(
+                f"with _bolt_runtime.memo.record(_bolt_runtime, {invocation}, __name__):"
+            )
+            with acc.block():
+                yield from visit_body(cast(AstRoot, body), acc)
+            if cached_identifiers:
+                acc.statement(f"{invocation}.bindings = ({cached_identifiers})")
+
+        return []
+
     @rule(AstCommand, identifier="def:function:body")
     def function(
         self,
@@ -501,26 +576,61 @@ class Codegen(Visitor):
         acc: Accumulator,
     ) -> Generator[AstNode, Optional[List[str]], Optional[List[str]]]:
         signature = cast(AstFunctionSignature, node.arguments[0])
+        body = cast(AstRoot, node.arguments[1])
 
         decorators: List[str] = []
         for decorator in signature.decorators:
             value = yield from visit_single(decorator.expression, required=True)
             decorators.append(value)
 
-        arguments: List[str] = [
-            f"{arg.name}={acc.missing()}" if arg.default else arg.name
-            for arg in signature.arguments
-        ]
-
-        with acc.function(signature.name, *arguments):
-            for arg in signature.arguments:
+        arguments: List[str] = []
+        for arg in signature.arguments:
+            if isinstance(arg, AstFunctionSignatureArgument):
+                a = arg.name
+                if arg.type_annotation:
+                    value = yield from visit_single(arg.type_annotation, required=True)
+                    a = f"{a}: {value}"
                 if arg.default:
+                    padding = " " * bool(arg.type_annotation)
+                    a = f"{a}{padding}={padding}{acc.missing()}"
+                arguments.append(a)
+                f"{arg.name}={acc.missing()}" if arg.default else arg.name
+            elif isinstance(arg, AstFunctionSignaturePositionalMarker):
+                arguments.append("/")
+            elif isinstance(arg, AstFunctionSignatureVariadicArgument):
+                a = f"*{arg.name}"
+                if arg.type_annotation:
+                    value = yield from visit_single(arg.type_annotation, required=True)
+                    a = f"{a}: {value}"
+                arguments.append(a)
+            elif isinstance(arg, AstFunctionSignatureVariadicMarker):
+                arguments.append("*")
+            elif isinstance(arg, AstFunctionSignatureVariadicKeywordArgument):
+                a = f"**{arg.name}"
+                if arg.type_annotation:
+                    value = yield from visit_single(arg.type_annotation, required=True)
+                    a = f"{a}: {value}"
+                arguments.append(a)
+
+        return_type = ""
+        if signature.return_type_annotation:
+            return_type = yield from visit_single(
+                signature.return_type_annotation, required=True
+            )
+
+        with acc.function(signature.name, *arguments, return_type=return_type):
+            if body.commands and isinstance(body.commands[0], AstDocstring):
+                yield body.commands[0]
+                body = replace(body, commands=AstChildren(body.commands[1:]))
+
+            for arg in signature.arguments:
+                if isinstance(arg, AstFunctionSignatureArgument) and arg.default:
                     acc.statement(f"if {arg.name} is {acc.missing()}:")
                     with acc.block():
                         value = yield from visit_single(arg.default, required=True)
                         acc.statement(f"{arg.name} = {value}")
 
-            yield from visit_body(cast(AstRoot, node.arguments[1]), acc)
+            yield from visit_body(body, acc)
 
         for decorator, value in list(zip(signature.decorators, decorators))[::-1]:
             acc.statement(
@@ -658,6 +768,15 @@ class Codegen(Visitor):
 
         return []
 
+    @rule(AstDocstring)
+    def docstring(
+        self,
+        node: AstDocstring,
+        acc: Accumulator,
+    ) -> Optional[List[str]]:
+        acc.statement(repr(cast(AstValue, node.arguments[0]).value))
+        return []
+
     @rule(AstCommand, identifier="del:target")
     def del_statement(
         self,
@@ -785,10 +904,12 @@ class Codegen(Visitor):
             alias = cast(AstImportedItem, node.arguments[1]).name
 
             if module.namespace:
+                full_path = module.get_value()
                 acc.statement(
-                    f"{alias} = _bolt_runtime.import_module({module.get_value()!r}).namespace",
+                    f"{alias} = _bolt_runtime.import_module({full_path!r}).namespace",
                     lineno=node,
                 )
+                acc.dependencies.add(full_path)
             else:
                 rhs = acc.import_module(module.path)
                 acc.statement(f"{alias} = {rhs}", lineno=node)
@@ -810,6 +931,9 @@ class Codegen(Visitor):
         node: AstFromImport,
         acc: Accumulator,
     ) -> Optional[List[str]]:
+        if isinstance(node, AstPrelude):
+            acc.prelude_imports.append(node)
+
         module = cast(AstResourceLocation, node.arguments[0])
         items = cast(
             AstChildren[Union[AstImportedItem, AstImportedMacro]], node.arguments[1:]
@@ -826,6 +950,7 @@ class Codegen(Visitor):
                     targets.append(item.name)
             stmt = f"{', '.join(targets)} = _bolt_runtime.from_module_import({full_path!r}, {', '.join(map(repr, names))})"
             acc.statement(stmt, lineno=node)
+            acc.dependencies.add(full_path)
         else:
             stmt = f"_bolt_from_import = {acc.import_module(module.path)}"
             acc.statement(stmt, lineno=node)
@@ -953,7 +1078,7 @@ class Codegen(Visitor):
     @rule(AstValue)
     def value(self, node: AstValue, acc: Accumulator) -> Optional[List[str]]:
         result = acc.make_variable()
-        acc.statement(f"{result} = {node.value!r}")
+        acc.statement(f"{result} = {node.literal}")
         return [result]
 
     @rule(AstIdentifier)
@@ -1131,6 +1256,19 @@ class Codegen(Visitor):
     ) -> Generator[AstNode, Optional[List[str]], Optional[List[str]]]:
         value = yield from visit_single(node.value, required=True)
         yield from visit_binding(node.target, node.operator, value, acc)
+        if node.type_annotation and isinstance(node.target, AstTargetIdentifier):
+            value = yield from visit_single(node.type_annotation, required=True)
+            acc.statement(f"{node.target.value}: {value}")
+        return []
+
+    @rule(AstTypeDeclaration)
+    def type_declaration(
+        self,
+        node: AstTypeDeclaration,
+        acc: Accumulator,
+    ) -> Generator[AstNode, Optional[List[str]], Optional[List[str]]]:
+        value = yield from visit_single(node.type_annotation, required=True)
+        acc.statement(f"{node.identifier.value}: {value}")
         return []
 
     @rule(AstTargetIdentifier)
