@@ -12,6 +12,7 @@ from .runtime import Runtime
 
 class BoltOptions(BaseModel):
     entrypoint: ListOption[str] = ListOption()
+    prelude: ListOption[str] = ListOption()
 
 
 def beet_default(ctx: Context):
@@ -23,3 +24,4 @@ def bolt(ctx: Context, opts: BoltOptions):
     """Plugin for configuring bolt."""
     runtime = ctx.inject(Runtime)
     runtime.evaluate.add_entrypoint(opts.entrypoint.entries())
+    runtime.modules.add_prelude(opts.prelude.entries())
